/*
 * Copyright (C) 2018 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. AND ITS CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL APPLE INC. OR ITS CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */

#pragma once

#if ENABLE(WEBGPU)

#include "DeferrableTask.h"
#include <wtf/RefCounted.h>
#include <wtf/RefPtr.h>
#include <wtf/RetainPtr.h>
#include <wtf/Vector.h>
#include <wtf/WeakPtr.h>

OBJC_PROTOCOL(MTLCommandQueue);

namespace WebCore {

using PlatformQueue = MTLCommandQueue;
using PlatformQueueSmartPtr = RetainPtr<MTLCommandQueue>;

class GPUCommandBuffer;
class GPUDevice;

class GPUQueue : public RefCounted<GPUQueue> {
public:
    static RefPtr<GPUQueue> tryCreate(const GPUDevice&);

    PlatformQueue* platformQueue() const { return m_platformQueue.get(); }

    void submit(Vector<Ref<GPUCommandBuffer>>&&);

    String label() const;
    void setLabel(const String&) const;

private:
    GPUQueue(PlatformQueueSmartPtr&&, const GPUDevice&);

    PlatformQueueSmartPtr m_platformQueue;
    WeakPtr<const GPUDevice> m_device;
    DeferrableTask<Timer> m_presentTask;
};

} // namespace WebCore

#endif // ENABLE(WEBGPU)
